@@ -95,7 +95,7 @@ case a_##act: \
 //'         \item conv() [v, shape] (convolve with vector v; shape="full" is equivalent
 //'            to R's \code{convolve(..., rev(v), type="open")}).
 //'         \item quantile() [p] (calculate quantiles corresponding to probabilities p;
-//'            equivalent to R's \code{quantile(..., probs=p, type=8)}).
+//'            equivalent to R's \code{quantile(..., probs=p, type=5)}).
 //'        }
 //'     }
 //' 
@@ -487,8 +487,10 @@ SEXP arrApply(NumericVector arr, unsigned int idim=1, std::string fun="sum", Lis
         break;
     }
     // rechape back the result
+//Rcout << "mv_res=" << mv_res << "\n";
     if (!mv_res)
         d=join_cols(d.head(idim-1), d.tail(d.size()-idim));
+//Rcout << "d=" << d << "\n";
 //Rprintf("yah5\n");
     RObject vres;
     if (logical_res) {
@@ -502,13 +504,43 @@ SEXP arrApply(NumericVector arr, unsigned int idim=1, std::string fun="sum", Lis
         if (d.size()) {
             d[idim-1]=cres.n_cols;
             //d.print("dim final");
+//Rcout << "dim=" << d << "\n";
             vres.attr("dim")=IntegerVector(d.begin(), d.end());
+            if (d.size() >= 1 && arr.hasAttribute("dimnames")) {
+                List tmp(arr.attr("dimnames"));
+                Nullable<CharacterVector> nm_i(tmp[idim-1]);
+                if (nm_i.isNull() || ((CharacterVector) nm_i).size() == cres.n_cols) {
+                    tmp=clone(tmp);
+                    vres.attr("dimnames")=tmp;
+                }
+            }
         } else {
+//Rcout << "null\n";
             vres.attr("dim")=R_NilValue;
         }
     } else {
         // reducing actions
         vres=wrap(res);
+        if (d.size()) {
+            //d[idim-1]=res.n_cols;
+            //d.print("dim final");
+//Rcout << "redu dim=" << d << "\n";
+            vres.attr("dim")=IntegerVector(d.begin(), d.end());
+            if (d.size() >= 1 && arr.hasAttribute("dimnames")) {
+                List tmp=arr.attr("dimnames");
+                tmp=clone(tmp);
+                // preserve (dim)names if any
+                if (d.size() == 1)
+                    vres.attr("names")=tmp[2-idim];
+                else {
+                    tmp.erase(idim-1);
+                    vres.attr("dimnames")=tmp;
+                }
+            }
+        } else {
+//Rcout << "null 2\n";
+            vres.attr("dim")=R_NilValue;
+        }
     }
     return vres;
 }
